--------------------------------------------------------------------------------
--------------------------------------------------------------------------------
--                                                                            --
--  COPYRIGHT (C) 2005, DILLON ENGINEERING, INC.                              --
--                                                                            --
--  Dillon Engineering, Inc.       http://www.dilloneng.com                   --
--  3925 West 50th St.             info@dilloneng.com                         --
--  Suite 202                      952-836-2413                               --
--  Edina, MN 55424                                                           --
--                                                                            --
--                                                                            --
--                                                                            --
--    Name        : tb_FpModules                                              --
--    Top Level   : tb_FpModules                                              --
--    Rev         : $Id$
--                                                                            --
--------------------------------------------------------------------------------
--------------------------------------------------------------------------------

-------------------------- Library Declarations --------------------------------
library work, ieee;
use ieee.std_logic_1164.all;
use std.textio.all;
use work.txt_util.all;

library de_fplic ;
use de_fplic.de_fplic_pkg.all ;

entity tb_FpModules is
  generic(
    path      : string  := "data/";

    Tpw_clk   : time := 10 ns
  );
end entity tb_FpModules;


architecture arch of tb_FpModules is

  -- Stimulus signals - signals mapped to the input and inout ports of tested entity
  signal add_a         : std_logic_vector(full_width-1 downto 0);
  signal add_b         : std_logic_vector(full_width-1 downto 0);
  signal sub_a         : std_logic_vector(full_width-1 downto 0);
  signal sub_b         : std_logic_vector(full_width-1 downto 0);
  signal mult_a        : std_logic_vector(full_width-1 downto 0);
  signal mult_b        : std_logic_vector(full_width-1 downto 0);
  signal div_a         : std_logic_vector(full_width-1 downto 0);
  signal div_b         : std_logic_vector(full_width-1 downto 0);
  signal sqrt_a        : std_logic_vector(full_width-1 downto 0);
  signal rcp_a         : std_logic_vector(full_width-1 downto 0);
  signal cmp_a         : std_logic_vector(full_width-1 downto 0);
  signal cmp_b         : std_logic_vector(full_width-1 downto 0);
  signal int2fp_a      : std_logic_vector(int_width-1 downto 0);
  signal fp2int_a      : std_logic_vector(full_width-1 downto 0);
  signal r             : std_logic;
  signal sync_in       : std_logic;
  signal sync_in_pre   : std_logic := '0';
  signal clk           : std_logic;

  -- Observed signals - signals mapped to the output ports of tested entity
  signal add_sync_out    : std_logic := '0';
  signal sub_sync_out    : std_logic := '0';
  signal mult_sync_out   : std_logic := '0';
  signal div_sync_out    : std_logic := '0';
  signal sqrt_sync_out   : std_logic := '0';
  signal rcp_sync_out    : std_logic := '0';
  signal cmp_sync_out    : std_logic := '0';
  signal int2fp_sync_out : std_logic := '0';
  signal fp2int_sync_out : std_logic := '0';
  signal fp2int_oflow    : std_logic := '0';
  signal add_x           : std_logic_vector(full_width-1 downto 0);
  signal sub_x           : std_logic_vector(full_width-1 downto 0);
  signal mult_x          : std_logic_vector(full_width-1 downto 0);
  signal div_x           : std_logic_vector(full_width-1 downto 0);
  signal sqrt_x          : std_logic_vector(full_width-1 downto 0);
  signal rcp_x           : std_logic_vector(full_width-1 downto 0);
  signal cmp_er          : std_logic := '0';
  signal cmp_eq          : std_logic := '0';
  signal cmp_lt          : std_logic := '0';
  signal cmp_gt          : std_logic := '0';
  signal int2fp_x        : std_logic_vector(full_width-1 downto 0);
  signal fp2int_x        : std_logic_vector(int_width-1 downto 0);
  signal add_sp_out      : std_logic_vector(3 downto 0);
  signal sub_sp_out      : std_logic_vector(3 downto 0);
  signal mult_sp_out     : std_logic_vector(3 downto 0);
  signal div_sp_out      : std_logic_vector(3 downto 0);
  signal sqrt_sp_out     : std_logic_vector(3 downto 0);
  signal rcp_sp_out      : std_logic_vector(3 downto 0);
  signal cmp_sp_out      : std_logic_vector(3 downto 0);
  signal int2fp_sp_out   : std_logic_vector(3 downto 0);
  signal fp2int_sp_out   : std_logic_vector(3 downto 0);

  -- simulation control
  signal addCompCount,subCompCount,multCompCount,divCompCount,
         sqrtCompCount,rcpCompCount,cmpCompCount,int2fpCompCount,
         fp2intCompCount : integer ;
  signal addDone,subDone,multDone,divDone,sqrtDone,
         rcpDone,int2fpDone,fp2intDone,cmpDone : std_logic := '0' ;

  -- ************************************************************************
  -- *  testbench internal signals, data etc.
  -- *
  
begin

  -- Device Under Test
  ADD : comp_add_FpAddSub_0
  port map (  
    port_a         => add_a,
    port_b         => add_b,
    port_sp_out    => add_sp_out,
    port_sync_in   => sync_in,
    port_clk       => clk,
    port_sync_out  => add_sync_out,
    port_x         => add_x
  );
 
  SUB : comp_sub_FpAddSub_0
  port map (  
    port_a         => sub_a,
    port_b         => sub_b,
    port_sp_out    => sub_sp_out,
    port_sync_in   => sync_in,
    port_clk       => clk,
    port_sync_out  => sub_sync_out,
    port_x         => sub_x
  );
 
  MULT : comp_mult_FpMult_0
  port map (  
    port_a         => mult_a,
    port_b         => mult_b,
    port_sp_out    => mult_sp_out,
    port_sync_in   => sync_in,
    port_clk       => clk,
    port_sync_out  => mult_sync_out,
    port_x         => mult_x
  );
 
  DIV : comp_div_FpDiv_0
  port map (  
    port_a         => div_a,
    port_b         => div_b,
    port_sp_out    => div_sp_out,
    port_sync_in   => sync_in,
    port_clk       => clk,
    port_sync_out  => div_sync_out,
    port_x         => div_x
  );
 
  SQRT : comp_sqrt_FpSqrt_0
  port map (  
    port_a         => sqrt_a,
    port_sp_out    => sqrt_sp_out,
    port_sync_in   => sync_in,
    port_clk       => clk,
    port_sync_out  => sqrt_sync_out,
    port_x         => sqrt_x
  );
 
  RCP : comp_rcp_FpRcp_0
  port map (  
    port_a         => rcp_a,
    port_sp_out    => rcp_sp_out,
    port_sync_in   => sync_in,
    port_clk       => clk,
    port_sync_out  => rcp_sync_out,
    port_x         => rcp_x
  );
 
  CMP : comp_cmp_FpCmp_0
  port map (  
    port_a         => cmp_a,
    port_b         => cmp_b,
    port_sync_in   => sync_in,
    port_clk       => clk,
    port_sync_out  => cmp_sync_out,
    port_er        => cmp_er,
    port_eq        => cmp_eq,
    port_lt        => cmp_lt,
    port_gt        => cmp_gt
  );
 
  INT2FP : comp_i2f_Int2Fp_0
  port map (  
    port_a         => int2fp_a,
    port_sync_in   => sync_in,
    port_clk       => clk,
    port_sync_out  => int2fp_sync_out,
    port_x         => int2fp_x
  );
 
  FP2INT : comp_f2i_Fp2Int_0
  port map (  
    port_a         => fp2int_a,
    port_sync_in   => sync_in,
    port_clk       => clk,
    port_sync_out  => fp2int_sync_out,
    port_oflow     => fp2int_oflow,
    port_x         => fp2int_x
  );
 
  -- clock generator
  clock_gen : process is
  begin
    clk   <= '1' after Tpw_clk, '0' after 2 * Tpw_clk;
    wait for 2 * Tpw_clk;
  end process clock_gen;

  reg : process(clk) is 
  begin
    if clk'event and clk = '0' then
      sync_in <= sync_in_pre ;
    end if;
  end process reg ;

  -- overall time frame of the input process
  input_timing : process is
  begin
    sync_in_pre <= '0' ;
    r           <= '0' ;
    -- reset the logic
    for count in 0 to 11 loop
      r <= '1' ;
      wait until clk'event and clk = '0';
    end loop;
    r <= '0' ;
    for count in 0 to 10 loop
      wait until clk'event and clk = '0';
    end loop;

    while True loop
      sync_in_pre <= '1' ;
      wait until clk'event and clk = '0';
      sync_in_pre <= '0' ;
      wait until clk'event and clk = '0';
      sync_in_pre <= '1' ;
      wait until clk'event and clk = '0';
      sync_in_pre <= '1' ;
      wait until clk'event and clk = '0';
      sync_in_pre <= '1' ;
      wait until clk'event and clk = '0';
      sync_in_pre <= '0' ;
      wait until clk'event and clk = '0';
      sync_in_pre <= '0' ;
      wait until clk'event and clk = '0';
    end loop;
  end process input_timing;


  -- stimulate a/b from file
  add_file_stim_in : process is
    
    file     iAddFp        : text;
    variable aLine         : line;
    variable aS,bS         : string (full_width downto 1);
    variable tmpS          : string (1 downto 1);
    
  begin
    
    file_open(iAddFp, path & "FpAdd_" & str(man_width) & "_" & str(exp_width) & "_in.txt",read_mode);

    while not endfile(iAddFp) loop
      
      wait until clk'event and clk = '0' ;
      if sync_in_pre = '1' then
        readline(iAddFp, aLine);
        read(aLine, aS);
        read(aLine, tmpS); -- get rid of space
        read(aLine, bS);
        add_a <= to_std_logic_vector(aS);
        add_b <= to_std_logic_vector(bS);
        -- report aS & "  " & bS ;
      end if;

    end loop;

    file_close(iAddFp);

    -- forever
    while True loop
      wait until clk'event and clk = '0' ;
    end loop ;

  end process add_file_stim_in;
  
  sub_file_stim_in : process is
    
    file     iSubFp : text;
    variable aLine         : line;
    variable aS,bS         : string (full_width downto 1);
    variable tmpS          : string (1 downto 1);
    
  begin
    
    file_open(iSubFp, path & "FpSub_" & str(man_width) & "_" & str(exp_width) & "_in.txt",read_mode);

    while not endfile(iSubFp) loop
      
      wait until clk'event and clk = '0' ;
      if sync_in_pre = '1' then
        readline(iSubFp, aLine);
        read(aLine, aS);
        read(aLine, tmpS); -- get rid of space
        read(aLine, bS);
        sub_a <= to_std_logic_vector(aS);
        sub_b <= to_std_logic_vector(bS);
        -- report aS & "  " & bS ;
      end if;

    end loop;

    file_close(iSubFp);

    -- forever
    while True loop
      wait until clk'event and clk = '0' ;
    end loop ;

  end process sub_file_stim_in;
  
  mult_file_stim_in : process is
    
    file     iFp   : text;
    variable aLine : line;
    variable aS,bS : string (full_width downto 1);
    variable tmpS  : string (1 downto 1);
    
  begin
    
    file_open(iFp, path & "FpMult_" & str(man_width) & "_" & str(exp_width) & "_in.txt",read_mode);

    while not endfile(iFp) loop
      
      wait until clk'event and clk = '0' ;
      if sync_in_pre = '1' then
        readline(iFp, aLine);
        read(aLine, aS);
        read(aLine, tmpS); -- get rid of space
        read(aLine, bS);
        mult_a <= to_std_logic_vector(aS);
        mult_b <= to_std_logic_vector(bS);
      end if;

    end loop;

    file_close(iFp);

    -- forever
    while True loop
      wait until clk'event and clk = '0' ;
    end loop ;

  end process mult_file_stim_in;

  div_file_stim_in : process is
    
    file     iFp   : text;
    variable aLine : line;
    variable aS,bS : string (full_width downto 1);
    variable tmpS  : string (1 downto 1);
    
  begin
    
    file_open(iFp, path & "FpDiv_" & str(man_width) & "_" & str(exp_width) & "_in.txt",read_mode);

    while not endfile(iFp) loop
      
      wait until clk'event and clk = '0' ;
      if sync_in_pre = '1' then
        readline(iFp, aLine);
        read(aLine, aS);
        read(aLine, tmpS); -- get rid of space
        read(aLine, bS);
        div_a <= to_std_logic_vector(aS);
        div_b <= to_std_logic_vector(bS);
      end if;

    end loop;

    file_close(iFp);

    -- forever
    while True loop
      wait until clk'event and clk = '0' ;
    end loop ;

  end process div_file_stim_in;

  sqrt_file_stim_in : process is
    
    file     iFp   : text;
    variable aLine : line;
    variable aS    : string (full_width downto 1);
    variable tmpS  : string (1 downto 1);
    
  begin
    
    file_open(iFp, path & "FpSqrt_" & str(man_width) & "_" & str(exp_width) & "_in.txt",read_mode);

    while not endfile(iFp) loop
      
      wait until clk'event and clk = '0' ;
      if sync_in_pre = '1' then
        readline(iFp, aLine);
        read(aLine, aS);
        sqrt_a <= to_std_logic_vector(aS);
      end if;

    end loop;

    file_close(iFp);

    -- forever
    while True loop
      wait until clk'event and clk = '0' ;
    end loop ;

  end process sqrt_file_stim_in;

  rcp_file_stim_in : process is
    
    file     iFp   : text;
    variable aLine : line;
    variable aS    : string (full_width downto 1);
    variable tmpS  : string (1 downto 1);
    
  begin
    
    file_open(iFp, path & "FpRcp_" & str(man_width) & "_" & str(exp_width) & "_in.txt",read_mode);

    while not endfile(iFp) loop
      
      wait until clk'event and clk = '0' ;
      if sync_in_pre = '1' then
        readline(iFp, aLine);
        read(aLine, aS);
        rcp_a <= to_std_logic_vector(aS);
      end if;

    end loop;

    file_close(iFp);

    -- forever
    while True loop
      wait until clk'event and clk = '0' ;
    end loop ;

  end process rcp_file_stim_in;

  cmp_file_stim_in : process is
    
    file     iFp   : text;
    variable aLine : line;
    variable aS,bS : string (full_width downto 1);
    variable tmpS  : string (1 downto 1);
    
  begin
    
    file_open(iFp, path & "FpCmp_" & str(man_width) & "_" & str(exp_width) & "_in.txt",read_mode);

    while not endfile(iFp) loop
      
      wait until clk'event and clk = '0' ;
      if sync_in_pre = '1' then
        readline(iFp, aLine);
        read(aLine, aS);
        read(aLine, tmpS); -- get rid of space
        read(aLine, bS);
        cmp_a <= to_std_logic_vector(aS);
        cmp_b <= to_std_logic_vector(bS);
      end if;

    end loop;

    file_close(iFp);

    -- forever
    while True loop
      wait until clk'event and clk = '0' ;
    end loop ;

  end process cmp_file_stim_in;

  int2fp_file_stim_in : process is
    
    file     iFp   : text;
    variable aLine : line;
    variable aS    : string (int_width downto 1);
    
  begin
    
    file_open(iFp, path & "Int2Fp_" & str(int_width) & "_" & str(man_width) & "_" & str(exp_width) & "_in.txt",read_mode);

    while not endfile(iFp) loop
      
      wait until clk'event and clk = '0' ;
      if sync_in_pre = '1' then
        readline(iFp, aLine);
        read(aLine, aS);
        int2fp_a <= to_std_logic_vector(aS);
      end if;

    end loop;

    file_close(iFp);

    -- forever
    while True loop
      wait until clk'event and clk = '0' ;
    end loop ;

  end process int2fp_file_stim_in;

  fp2int_file_stim_in : process is
    
    file     iFp   : text;
    variable aLine : line;
    variable aS    : string (full_width downto 1);
    
  begin
    
    file_open(iFp, path & "Fp2Int_" & str(int_width) & "_" & str(man_width) & "_" & str(exp_width) & "_in.txt",read_mode);

    while not endfile(iFp) loop
      
      wait until clk'event and clk = '0' ;
      if sync_in_pre = '1' then
        readline(iFp, aLine);
        read(aLine, aS);
        fp2int_a <= to_std_logic_vector(aS);
      end if;

    end loop;

    file_close(iFp);

    -- forever
    while True loop
      wait until clk'event and clk = '0' ;
    end loop ;

  end process fp2int_file_stim_in;

  -- compare to expected results
  add_file_out_comp : process is
    
    file     oFp         : text;
    variable xLine       : line;
    variable xS          : string (full_width downto 1);
    variable tmpC        : character;
    variable sS          : string (4 downto 1);
  begin
    

    file_open(oFp, path & "FpAdd_" & str(man_width) & "_" & str(exp_width) & "_out.txt",read_mode);

    AddCompCount <= 0;
    while not endfile(oFp) loop
      wait until clk'event and clk = '0' ;
      if add_sync_out = '1' then
        readline(oFp, xLine);
        read(xLine, xS);
        read(xLine, tmpC);
        read(xLine, sS);


        assert to_std_logic_vector(xS) = add_x
          report "FpAdd x ERROR found 0x" & hstr(add_x) & ", expected: 0x" & hstr(to_std_logic_vector(xS)) & " at point " & str(AddCompCount)
          severity FAILURE;
        assert to_std_logic_vector(sS) = add_sp_out
          report "FpAdd sp_out ERROR found 0x" & hstr(add_sp_out) & ", expected: 0x" & hstr(to_std_logic_vector(sS)) & " at point " & str(AddCompCount)
          severity FAILURE;
        AddCompCount <= AddCompCount + 1;
      end if ;
    end loop;

    file_close(oFp);
    addDone <= '1' ;
    while True loop
      wait until clk'event and clk = '0' ;
    end loop;

  end process add_file_out_comp;

  sub_file_out_comp : process is
    
    file     oFp         : text;
    variable xLine       : line;
    variable xS          : string (full_width downto 1);
    variable tmpC        : character;
    variable sS          : string (4 downto 1);
  begin
    

    file_open(oFp, path & "FpSub_" & str(man_width) & "_" & str(exp_width) & "_out.txt",read_mode);

    subCompCount <= 0;
    while not endfile(oFp) loop
      wait until clk'event and clk = '0' ;
      if sub_sync_out = '1' then
        readline(oFp, xLine);
        read(xLine, xS);
        read(xLine, tmpC);
        read(xLine, sS);


        assert to_std_logic_vector(xS) = sub_x
          report "FpSub x ERROR found 0x" & hstr(sub_x) & ", expected: 0x" & hstr(to_std_logic_vector(xS)) & " at point " & str(subCompCount)
          severity FAILURE;
        assert to_std_logic_vector(sS) = sub_sp_out
          report "FpSub sp_out ERROR found 0x" & hstr(sub_sp_out) & ", expected: 0x" & hstr(to_std_logic_vector(sS)) & " at point " & str(subCompCount)
          severity FAILURE;
        subCompCount <= subCompCount + 1;
      end if ;
    end loop;

    file_close(oFp);
    subDone <= '1' ;
    while True loop
      wait until clk'event and clk = '0' ;
    end loop;

  end process sub_file_out_comp;
  
  mult_file_out_comp : process is
    
    file     oFp         : text;
    variable xLine       : line;
    variable xS          : string (full_width downto 1);
    variable tmpC        : character;
    variable sS          : string (4 downto 1);
  begin
    

    file_open(oFp, path & "FpMult_" & str(man_width) & "_" & str(exp_width) & "_out.txt",read_mode);

    multCompCount <= 0;
    while not endfile(oFp) loop
      wait until clk'event and clk = '0' ;
      if mult_sync_out = '1' then
        readline(oFp, xLine);
        read(xLine, xS);
        read(xLine, tmpC);
        read(xLine, sS);


        assert to_std_logic_vector(xS) = mult_x
          report "FpMult x ERROR found 0x" & hstr(mult_x) & ", expected: 0x" & hstr(to_std_logic_vector(xS)) & " at point " & str(multCompCount)
          severity FAILURE;
        assert to_std_logic_vector(sS) = mult_sp_out
          report "FpMult sp_out ERROR found 0x" & hstr(mult_sp_out) & ", expected: 0x" & hstr(to_std_logic_vector(sS)) & " at point " & str(multCompCount)
          severity FAILURE;
        multCompCount <= multCompCount + 1;
      end if ;
    end loop;

    file_close(oFp);
    multDone <= '1' ;
    while True loop
      wait until clk'event and clk = '0' ;
    end loop;

  end process mult_file_out_comp;
  
  div_file_out_comp : process is
    
    file     oFp         : text;
    variable xLine       : line;
    variable xS          : string (full_width downto 1);
    variable tmpC        : character;
    variable sS          : string (4 downto 1);
  begin
    

    file_open(oFp, path & "FpDiv_" & str(man_width) & "_" & str(exp_width) & "_out.txt",read_mode);

    divCompCount <= 0;
    while not endfile(oFp) loop
      wait until clk'event and clk = '0' ;
      if div_sync_out = '1' then
        readline(oFp, xLine);
        read(xLine, xS);
        read(xLine, tmpC);
        read(xLine, sS);

        assert to_std_logic_vector(xS) = div_x
          report "FpDiv x ERROR found 0x" & hstr(div_x) & ", expected: 0x" & hstr(to_std_logic_vector(xS)) & " at point " & str(divCompCount)
          severity FAILURE;
        assert to_std_logic_vector(sS) = div_sp_out
          report "FpDiv sp_out ERROR found 0x" & hstr(div_sp_out) & ", expected: 0x" & hstr(to_std_logic_vector(sS)) & " at point " & str(divCompCount)
          severity FAILURE;
        divCompCount <= divCompCount + 1;
      end if ;
    end loop;

    file_close(oFp);
    divDone <= '1' ;
    while True loop
      wait until clk'event and clk = '0' ;
    end loop;

  end process div_file_out_comp;

  sqrt_file_out_comp : process is
    
    file     oFp         : text;
    variable xLine       : line;
    variable xS          : string (full_width downto 1);
    variable tmpC        : character;
    variable sS          : string (4 downto 1);
  begin
    

    file_open(oFp, path & "FpSqrt_" & str(man_width) & "_" & str(exp_width) & "_out.txt",read_mode);

    sqrtCompCount <= 0;
    while not endfile(oFp) loop
      wait until clk'event and clk = '0' ;
      if sqrt_sync_out = '1' then
        readline(oFp, xLine);
        read(xLine, xS);
        read(xLine, tmpC);
        read(xLine, sS);

        assert to_std_logic_vector(xS) = sqrt_x
          report "FpSqrt x ERROR found 0x" & hstr(sqrt_x) & ", expected: 0x" & hstr(to_std_logic_vector(xS)) & " at point " & str(sqrtCompCount)
          severity FAILURE;
        assert to_std_logic_vector(sS) = sqrt_sp_out
          report "FpSqrt sp_out ERROR found 0x" & hstr(sqrt_sp_out) & ", expected: 0x" & hstr(to_std_logic_vector(sS)) & " at point " & str(sqrtCompCount)
          severity FAILURE;
        sqrtCompCount <= sqrtCompCount + 1;
      end if ;
    end loop;

    file_close(oFp);
    sqrtDone <= '1' ;
    while True loop
      wait until clk'event and clk = '0' ;
    end loop;

  end process sqrt_file_out_comp;

  rcp_file_out_comp : process is
    
    file     oFp         : text;
    variable xLine       : line;
    variable xS          : string (full_width downto 1);
    variable tmpC        : character;
    variable sS          : string (4 downto 1);
  begin
    

    file_open(oFp, path & "FpRcp_" & str(man_width) & "_" & str(exp_width) & "_out.txt",read_mode);

    rcpCompCount <= 0;
    while not endfile(oFp) loop
      wait until clk'event and clk = '0' ;
      if rcp_sync_out = '1' then
        readline(oFp, xLine);
        read(xLine, xS);
        read(xLine, tmpC);
        read(xLine, sS);

        assert to_std_logic_vector(xS) = rcp_x
          report "FpSqrt x ERROR found 0x" & hstr(rcp_x) & ", expected: 0x" & hstr(to_std_logic_vector(xS)) & " at point " & str(rcpCompCount)
          severity FAILURE;
        assert to_std_logic_vector(sS) = rcp_sp_out
          report "FpSqrt sp_out ERROR found 0x" & hstr(rcp_sp_out) & ", expected: 0x" & hstr(to_std_logic_vector(sS)) & " at point " & str(rcpCompCount)
          severity FAILURE;
        rcpCompCount <= rcpCompCount + 1;
      end if ;
    end loop;

    file_close(oFp);
    rcpDone <= '1' ;
    while True loop
      wait until clk'event and clk = '0' ;
    end loop;

  end process rcp_file_out_comp;

  cmp_file_out_comp : process is
    
    file     oFp         : text;
    variable xLine       : line;
    variable tmpS,erS,eqS,gtS,ltS : character ;
  begin
    

    file_open(oFp, path & "FpCmp_" & str(man_width) & "_" & str(exp_width) & "_out.txt",read_mode);

    cmpCompCount <= 0;
    while not endfile(oFp) loop
      wait until clk'event and clk = '0' ;
      if cmp_sync_out = '1' then
        readline(oFp, xLine);
        read(xLine, erS);
        read(xLine, tmpS); -- get rid of space
        read(xLine, eqS);
        read(xLine, tmpS); -- get rid of space
        read(xLine, gtS);
        read(xLine, tmpS); -- get rid of space
        read(xLine, ltS);
        read(xLine, tmpS); -- get rid of space


        assert to_std_logic(erS) = cmp_er
          report "FpCmp er ERROR found " & str(cmp_er) & ", expected: " & erS & " at point " & str(cmpCompCount) 
          severity FAILURE;
        assert to_std_logic(eqS) = cmp_eq
          report "FpCmp eq ERROR found " & str(cmp_eq) & ", expected: " & eqS & " at point " & str(cmpCompCount)
          severity FAILURE;
        assert to_std_logic(gtS) = cmp_gt
          report "FpCmp gt ERROR found " & str(cmp_gt) & ", expected: " & gtS & " at point " & str(cmpCompCount)
          severity FAILURE;
        assert to_std_logic(ltS) = cmp_lt
          report "FpCmp lt ERROR found " & str(cmp_lt) & ", expected: " & ltS & " at point " & str(cmpCompCount)
          severity FAILURE;
        cmpCompCount <= cmpCompCount + 1;
      end if ;
    end loop;

    file_close(oFp);
    cmpDone <= '1' ;
    while True loop
      wait until clk'event and clk = '0' ;
    end loop;

  end process cmp_file_out_comp;

  int2fp_file_out_comp : process is
    
    file     oFp         : text;
    variable xLine       : line;
    variable xS          : string (full_width downto 1);
  begin
    

    file_open(oFp, path & "Int2Fp_" & str(int_width) & "_" & str(man_width) & "_" & str(exp_width) & "_out.txt",read_mode);

    int2fpCompCount <= 0;
    while not endfile(oFp) loop
      wait until clk'event and clk = '0' ;
      if int2fp_sync_out = '1' then
        readline(oFp, xLine);
        read(xLine, xS);

        assert to_std_logic_vector(xS) = int2fp_x
          report "Int2Fp ERROR found 0x" & hstr(int2fp_x) & ", expected: 0x" & hstr(to_std_logic_vector(xS)) & " at point " & str(int2fpCompCount)
          severity FAILURE;
        int2fpCompCount <= int2fpCompCount + 1;
      end if ;
    end loop;

    file_close(oFp);
    int2fpDone <= '1' ;
    while True loop
      wait until clk'event and clk = '0' ;
    end loop;

  end process int2fp_file_out_comp;

  fp2int_file_out_comp : process is
    
    file     oFp         : text;
    variable xLine       : line;
    variable xS          : string (int_width downto 1);
    variable tmpC,oC     : character ;
  begin
    

    file_open(oFp, path & "Fp2Int_" & str(int_width) & "_" & str(man_width) & "_" & str(exp_width) & "_out.txt",read_mode);

    fp2intCompCount <= 0;
    while not endfile(oFp) loop
      wait until clk'event and clk = '0' ;
      if fp2int_sync_out = '1' then
        readline(oFp, xLine);
        read(xLine, xS);
        read(xLine, tmpC);
        read(xLine, oC);

        assert to_std_logic_vector(xS) = fp2int_x
          report "Fp2Int x ERROR found 0x" & hstr(fp2int_x) & ", expected: 0x" & hstr(to_std_logic_vector(xS)) & " at point " & str(fp2intCompCount)
          severity FAILURE;
        assert to_std_logic(oC) = fp2int_oflow
          report "Fp2Int oflow ERROR found " & str(fp2int_oflow) & ", expected: " & oC & " at point " & str(fp2intCompCount)
          severity FAILURE;
        fp2intCompCount <= fp2intCompCount + 1;
      end if ;
    end loop;

    file_close(oFp);
    fp2intDone <= '1' ;
    while True loop
      wait until clk'event and clk = '0' ;
    end loop;

  end process fp2int_file_out_comp;

  simdone : process(clk) is 
  begin
    if clk'event and clk = '1' then
      if addDone = '1' and subDone = '1' and multDone = '1' and divDone = '1' and 
         int2fpDone = '1' and fp2intDone = '1' and cmpDone = '1' and sqrtDone = '1' and rcpDone = '1' then
        report "Finished simulation, " & integer'image(addCompCount+subCompCount+multCompCount+divCompCount+sqrtCompCount+rcpCompCount+int2fpCompCount+fp2intCompCount+cmpCompCount) & " points compared OK, 0 Errors" 
          severity Warning;
      end if;
    end if;
  end process simdone ;

  
end architecture arch;
